import heapq
from collections import defaultdict
from typing import Any, Iterator, TypeAlias

from ..base import BaseSolver

Position: TypeAlias = tuple[int, int]
Direction: TypeAlias = tuple[int, int]


class Solver(BaseSolver):
    def solve(self, input: str) -> Iterator[Any]:
        grid = [list(r) for r in input.splitlines()]
        n_rows, n_cols = len(grid), len(grid[0])
        rein = next(
            (i, j) for i in range(n_rows) for j in range(n_cols) if grid[i][j] == "S"
        )
        target = next(
            (i, j) for i in range(n_rows) for j in range(n_cols) if grid[i][j] == "E"
        )

        queue: list[tuple[int, Position, Direction, tuple[Position, ...]]] = [
            (0, rein, (0, 1), (rein,))
        ]

        max_score = n_rows * n_cols * 1000
        target_score: int = max_score
        scores: dict[tuple[Position, Direction], int] = defaultdict(lambda: max_score)
        visited: set[Position] = set()

        while queue:
            score, pos, dir, path = heapq.heappop(queue)

            if target_score < score:
                break

            if pos == target:
                target_score = score
                visited |= set(path)
                continue

            scores[pos, dir] = score

            row, col = pos
            d_row, d_col = dir

            for cost, n_pos, n_dir in (
                (1, (row + d_row, col + d_col), dir),
                (1000, pos, (1, 0) if d_row == 0 else (0, 1)),
                (1000, pos, (-1, 0) if d_row == 0 else (0, -1)),
            ):
                n_row, n_col = n_pos
                score_n = score + cost
                if grid[n_row][n_col] != "#" and score_n < scores[n_pos, n_dir]:
                    heapq.heappush(
                        queue,
                        (score_n, n_pos, n_dir, path + (n_pos,)),
                    )

        assert target_score is not None
        yield target_score
        yield len(visited)
