import os
from pathlib import Path
from typing import Final

from ...base import FileHandler
from .base import dump_api_message


class FileHandlerAPI(FileHandler):
    def __init__(self, folder: Path):
        self.folder: Final = folder

    def make_path(self, filename: str) -> Path:
        return self.folder.joinpath(filename)

    def notify_created(self, path: Path):
        dump_api_message("file", {"filename": path.name, "size": os.stat(path).st_size})

    def _create(self, path: Path, content: bytes, text: bool = False):
        self.folder.mkdir(exist_ok=True)
        with open(path, "wb") as fp:
            fp.write(content)
        return path
